@@ -120,6 +120,9 @@ class Capture(object):
                 imports.append(trigger)
         self._import(imports)
 
+    def reset_capture_request(self):
+        self._capture_request = None
+    
     def _config_missing_pallete(self, cap_filter, pallette, trigger, filter):
         pallete_map = getattr(self, '_{0}_OFFSET_MAP'.format(
                                 cap_filter.parent.choice.upper()))
@@ -250,7 +253,8 @@ class Capture(object):
         """Gets capture file and returns it as a byte stream
         """
         with Timer(self._api, 'Captures stop'):
-            capture = self._api._vport.find(Name=request.port_name).Capture
+            capture = self._api._vport.find(Name=self._api.special_char(
+                            request.port_name)).Capture
             capture.Stop('allTraffic')
 
             #   Internally setting max time_out to 90sec with 3sec polling interval.
@@ -260,7 +264,8 @@ class Capture(object):
             for x in range(retry_count):
                 port_ready = True
                 time.sleep(3)
-                capture = self._api._vport.find(Name=request.port_name).Capture
+                capture = self._api._vport.find(Name=self._api.special_char(
+                                request.port_name)).Capture
                 if capture.HardwareEnabled and capture.DataCaptureState == 'notReady':
                     port_ready = False
                     continue
