@@ -18,9 +18,11 @@ class TrafficItem(CustomField):
         ('bytes_tx', 'Tx Bytes', int),
         ('bytes_rx', 'Rx Bytes', int),
         ('loss', 'Loss %', float),
-        # TODO: these are not defined in API spec
         ('bytes_tx_rate', 'Tx Rate (Bps)', float),
         ('bytes_rx_rate', 'Rx Rate (Bps)', float),
+        ('min_latency_ns', 'Store-Forward Min Latency (ns)', int),
+        ('max_latency_ns', 'Store-Forward Max Latency (ns)', int),
+        ('avg_latency_ns', 'Store-Forward Avg Latency (ns)', int)
     ]
 
     _STACK_IGNORE = ['ethernet.fcs', 'pfcPause.fcs']
@@ -190,7 +192,7 @@ class TrafficItem(CustomField):
                     'TrafficType': self._get_traffic_type(flow),
                     'SrcDestMesh': self._get_mesh_type(flow)
                 }
-                ixn_traffic_item.find(Name='^%s$' % flow.name,
+                ixn_traffic_item.find(Name='^%s$' % self._api.special_char(flow.name),
                                       TrafficType=args['TrafficType'])
                 if len(ixn_traffic_item) == 0:
                     ixn_traffic_item.add(**args)
@@ -420,25 +422,26 @@ class TrafficItem(CustomField):
                              ValueType='valueList',
                              ValueList=pattern.values)
         elif pattern.choice == 'increment':
-            try:
-                ixn_field.update(Auto=False,
-                                 ValueType='increment',
-                                 ActiveFieldChoice=field_choice,
-                                 StartValue=pattern.increment.start,
-                                 StepValue=pattern.increment.step,
-                                 CountValue=pattern.increment.count)
-            except Exception as e:
-                print(e)
+            # try:
+            ixn_field.update(Auto=False,
+                            ValueType='increment',
+                            ActiveFieldChoice=field_choice,
+                            StartValue=pattern.increment.start,
+                            StepValue=pattern.increment.step,
+                            CountValue=pattern.increment.count)
+            # except Exception as e:
+            #     import pdb; pdb.set_trace()
+            #     print(e)
         elif pattern.choice == 'decrement':
-            try:
-                ixn_field.update(Auto=False,
-                                 ValueType='decrement',
-                                 ActiveFieldChoice=field_choice,
-                                 StartValue=pattern.decrement.start,
-                                 StepValue=pattern.decrement.step,
-                                 CountValue=pattern.decrement.count)
-            except Exception as e:
-                print(e)
+            # try:
+            ixn_field.update(Auto=False,
+                            ValueType='decrement',
+                            ActiveFieldChoice=field_choice,
+                            StartValue=pattern.decrement.start,
+                            StepValue=pattern.decrement.step,
+                            CountValue=pattern.decrement.count)
+            # except Exception as e:
+            #     print(e)
         elif pattern.choice == 'random':
             ixn_field.update(Auto=False,
                              ActiveFieldChoice=field_choice,
@@ -516,6 +519,20 @@ class TrafficItem(CustomField):
         args['Rate'] = value
         self._update(ixn_frame_rate, **args)
 
+    def _configure_delay(self, delay, args):
+        if delay.choice is not None:
+            value = getattr(delay, delay.choice, None)
+            if value is None:
+                raise Exception("Delay must be of type <int>")
+            if isinstance(value, float):
+                self._api.warning("Cast Delay to <int> due to software limitation")
+            if delay.choice == 'microseconds':
+                args['StartDelayUnits'] = 'nanoseconds'
+                args['StartDelay'] = value * 1000
+            else:
+                args['StartDelayUnits'] = delay.choice
+                args['StartDelay'] = value
+    
     def _configure_tx_control(self, ixn_stream, hl_stream_count, duration):
         """Transform duration flows.duration to /traffic/trafficItem[*]/configElement[*]/TransmissionControl
         """
@@ -526,42 +543,36 @@ class TrafficItem(CustomField):
         if duration.choice == 'continuous':
             args['Type'] = 'continuous'
             args['MinGapBytes'] = duration.continuous.gap
-            args['StartDelay'] = duration.continuous.delay
-            args['StartDelayUnits'] = duration.continuous.delay_unit
+            self._configure_delay(duration.continuous.delay, args)
         elif duration.choice == 'fixed_packets':
             args['Type'] = 'fixedFrameCount'
             args['FrameCount'] = duration.fixed_packets.packets / hl_stream_count
             args['MinGapBytes'] = duration.fixed_packets.gap
-            args['StartDelay'] = duration.fixed_packets.delay
-            args['StartDelayUnits'] = duration.fixed_packets.delay_unit
+            self._configure_delay(duration.fixed_packets.delay, args)
         elif duration.choice == 'fixed_seconds':
             args['Type'] = 'fixedDuration'
             args['Duration'] = duration.fixed_seconds.seconds
             args['MinGapBytes'] = duration.fixed_seconds.gap
-            args['StartDelay'] = duration.fixed_seconds.delay
-            args['StartDelayUnits'] = duration.fixed_seconds.delay_unit
+            self._configure_delay(duration.fixed_seconds.delay, args)
         elif duration.choice == 'burst':
             args['Type'] = 'custom'
             args['BurstPacketCount'] = duration.burst.packets
             args['MinGapBytes'] = duration.burst.gap
             args[
                 'EnableInterBurstGap'] = True if duration.burst.gap > 0 else False
-            args['InterBurstGap'] = duration.burst.inter_burst_gap
-            args['InterBurstGapUnits'] = duration.burst.inter_burst_gap_unit
+            inter_burst_gap = duration.burst.inter_burst_gap
+            if inter_burst_gap.choice is not None:
+                value = getattr(inter_burst_gap, inter_burst_gap.choice, None)
+                if value is None:
+                    raise Exception("Inter packet gap mush be of type some <int> value")
+                if inter_burst_gap.choice == 'microseconds':
+                    args['InterBurstGap'] = value * 1000
+                    args['InterBurstGapUnits'] = 'nanoseconds'
+                else:
+                    args['InterBurstGap'] = value
+                    args['InterBurstGapUnits'] = inter_burst_gap.choice
         self._update(ixn_tx_control, **args)
 
-    def _convert_string_to_regex(self, names):
-        ret_list = []
-        for n in names:
-            ret_list.append(
-                n.replace('(', '\\(').replace(')', '\\)')
-                    .replace('[', '\\[').replace(']', '\\]')
-                    .replace('.', '\\.').replace('*', '\\*')
-                    .replace('+', '\\+').replace('?', '\\?')
-                    .replace('{', '\\{').replace('}', '\\}')
-            )
-        return ret_list
-
     def transmit(self, request):
         """Set flow transmit
         1) If start then start any device protocols that are traffic dependent
@@ -573,10 +584,10 @@ class TrafficItem(CustomField):
         if request and request.flow_names:
             flow_names = request.flow_names
         if len(flow_names) == 1:
-            regex = '^%s$' % self._convert_string_to_regex(flow_names)[0]
+            regex = '^%s$' % self._api.special_char(flow_names)[0]
         elif len(flow_names) > 1:
             regex = '^(%s)$' % '|'.join(
-                self._convert_string_to_regex(flow_names)
+                self._api.special_char(flow_names)
             )
 
         if request.state == 'start':
@@ -600,7 +611,8 @@ class TrafficItem(CustomField):
         self._api._traffic_item.find(Name=regex)
         if len(self._api._traffic_item) > 0:
             if request.state == 'start':
-                self._api._traffic_item.find(Name=regex, State='^started$')
+                self._api._traffic_item.find(Name=regex, Suspend=True,
+                                             State='^started$')
                 if len(self._api._traffic_item) > 0:
                     with Timer(self._api, 'Flows resume'):
                         self._api._traffic_item.PauseStatelessTraffic(False)
@@ -677,8 +689,7 @@ class TrafficItem(CustomField):
 
         filter = {'property': 'name', 'regex': '.*'}
         filter['regex'] = '^(%s)$' % '|'.join(
-            self._convert_string_to_regex(flow_names)
-        )
+                        self._api.special_char(flow_names))
 
         flow_count = len(flow_names)
         ixn_page = self._api._ixnetwork.Statistics.View.find(Caption="Flow Statistics").Page
